import { Observable, Subject } from 'rxjs';
import { InjectionToken, Injectable, Optional, Inject, NgZone, Directive, ElementRef, Input, Output, EventEmitter, HostListener, NgModule } from '@angular/core';
import { shareReplay, take, map, takeUntil, tap, debounceTime, filter, distinctUntilChanged } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Create an observable from a disposable.
 * @template T
 * @param {?} source
 * @return {?}
 */
function fromDisposable(source) {
    return Observable.create((observer) => {
        const /** @type {?} */ disposable = source(e => {
            observer.next(e);
        });
        return () => disposable.dispose();
    });
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Use this token to inject custom completion providers.
 */
const COMPLETION_PROVIDERS = new InjectionToken('Completion Provider');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Use this token to inject the default monaco editor options.
 */
const MONACO_EDITOR_OPTIONS = new InjectionToken('Monaco Editor Options');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class MonacoEditorService {
    /**
     * @param {?} completionProviders
     * @param {?} editorOptions
     * @param {?} zone
     */
    constructor(completionProviders, editorOptions, zone) {
        this.completionProviders = completionProviders;
        this.editorOptions = editorOptions;
        this.zone = zone;
        this.onFileChange = new Subject();
        this.fileChange$ = this.onFileChange.asObservable();
        this.bootstrap$ = Observable.create((observer) => {
            const /** @type {?} */ script = document.createElement('script');
            script.type = 'text/javascript';
            script.src = 'libs/vs/loader.js';
            script.onload = () => {
                window.require.config({ paths: { vs: 'libs/vs' } });
                window.require(['vs/editor/editor.main'], () => {
                    // Emit that we are ready
                    observer.next();
                });
            };
            // Add the script tag to the page in order to start loading monaco
            document.body.appendChild(script);
        }).pipe(shareReplay(1));
    }
    /**
     * @return {?}
     */
    get editor() {
        return this.monacoEditor;
    }
    /**
     * @return {?}
     */
    registerCompletionProviders() {
        if (!this.completionProviders) {
            return;
        }
        // Register all the completion providers
        for (const /** @type {?} */ completionProvider of this.completionProviders) {
            monaco.languages.registerCompletionItemProvider(completionProvider.language, completionProvider);
        }
    }
    /**
     * @param {?} file
     * @param {?} model
     * @return {?}
     */
    registerModelChangeListener(file, model) {
        const /** @type {?} */ destroy = fromDisposable(model.onWillDispose.bind(model)).pipe(take(1));
        // Subscribe to changes from the model
        fromDisposable(model.onDidChangeContent.bind(model))
            .pipe(map(() => model.getValue()), takeUntil(destroy))
            .subscribe((content) => {
            this.zone.run(() => {
                this.onFileChange.next(Object.assign({}, file, { content }));
            });
        });
    }
    /**
     * Dispose all editor models
     * @return {?}
     */
    disposeModels() {
        if (!window.monaco) {
            return;
        }
        for (const /** @type {?} */ model of monaco.editor.getModels()) {
            model.dispose();
        }
    }
    /**
     * Bootstrap the monaco editor.
     *
     * @param {?} container Container of the editor.
     * @param {?=} options Editor options.
     * @return {?}
     */
    load(container, options = {}) {
        const /** @type {?} */ editorOptions = options.editor || this.editorOptions || {};
        return this.bootstrap$.pipe(tap(() => {
            // Dispose all the current models
            this.disposeModels();
            // Create a new monaco editor
            this.monacoEditor = monaco.editor.create(container.nativeElement, Object.assign({ theme: options.theme }, editorOptions));
            // Register the completion providers
            this.registerCompletionProviders();
            // Open the file
            if (this.file) {
                this.open(this.file);
            }
        }));
    }
    /**
     * Open the provided file with the editor.
     *
     * @param {?} file File to open.
     * @return {?}
     */
    open(file) {
        this.file = file;
        if (!this.monacoEditor) {
            // Exit early if the editor is not bootstrapped yet. It will automatically open the provided file when ready.
            return;
        }
        const /** @type {?} */ uri = monaco.Uri.file(file.uri);
        let /** @type {?} */ model = monaco.editor.getModel(uri);
        if (model) {
            if (file.language && model.getModeId() !== file.language) {
                model.dispose();
                model = undefined;
            }
            else {
                model.setValue(file.content);
            }
        }
        if (!model) {
            model = monaco.editor.createModel(file.content, file.language, uri);
            // Listen for changes in the model
            this.registerModelChangeListener(file, model);
        }
        this.monacoEditor.setModel(model);
    }
    /**
     * Change the options of the editor.
     *
     * @param {?} options Editor options.
     * @return {?}
     */
    updateOptions(options) {
        if (this.monacoEditor) {
            this.monacoEditor.updateOptions(options);
        }
    }
    /**
     * Change the theme of the monaco editor.
     *
     * @param {?} theme Name of the editor theme.
     * @return {?}
     */
    setTheme(theme) {
        if (window.monaco) {
            monaco.editor.setTheme(theme);
        }
    }
}
MonacoEditorService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
MonacoEditorService.ctorParameters = () => [
    { type: Array, decorators: [{ type: Optional }, { type: Inject, args: [COMPLETION_PROVIDERS,] },] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [MONACO_EDITOR_OPTIONS,] },] },
    { type: NgZone, },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class MonacoEditorDirective {
    /**
     * @param {?} monacoEditorService
     * @param {?} editorRef
     */
    constructor(monacoEditorService, editorRef) {
        this.monacoEditorService = monacoEditorService;
        this.editorRef = editorRef;
        // Outputs
        this.ready = new EventEmitter();
        this.fileChange = new EventEmitter();
        this.resize$ = new Subject();
        this.destroy$ = new Subject();
    }
    /**
     * @return {?}
     */
    onResize() {
        this.resize$.next();
    }
    /**
     * Open the provided file with the editor.
     *
     * @param {?} file File to open.
     * @return {?}
     */
    open(file) {
        this.monacoEditorService.open(file);
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        // Load the monaco editor
        this.monacoEditorService.load(this.editorRef, {
            theme: this.theme,
            editor: this.options
        }).subscribe(() => {
            this.ready.emit();
        });
        // Listen for file changes
        this.monacoEditorService.fileChange$.pipe(takeUntil(this.destroy$)).subscribe(file => {
            this.fileChange.emit(file);
        });
        // Resize the editor when the window resizes
        this.resize$.pipe(filter(() => Boolean(this.monacoEditorService.editor)), map(() => ({ width: this.editorRef.nativeElement.clientWidth, height: this.editorRef.nativeElement.clientHeight })), distinctUntilChanged((a, b) => a.width === b.width && a.height === b.height), debounceTime(50), takeUntil(this.destroy$)).subscribe(dimension => {
            this.monacoEditorService.editor.layout(dimension);
        });
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.destroy$.next();
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes["options"]) {
            // Update the monaco editor options
            this.monacoEditorService.updateOptions(changes["options"].currentValue);
        }
        if (changes["theme"]) {
            // Update the theme
            this.monacoEditorService.setTheme(changes["theme"].currentValue);
        }
        if (changes["file"]) {
            // Open the new file
            this.open(changes["file"].currentValue);
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewChecked() {
        this.resize$.next();
    }
}
MonacoEditorDirective.decorators = [
    { type: Directive, args: [{
                selector: 'monaco-editor,[monaco-editor]'
            },] },
];
/** @nocollapse */
MonacoEditorDirective.ctorParameters = () => [
    { type: MonacoEditorService, },
    { type: ElementRef, },
];
MonacoEditorDirective.propDecorators = {
    "theme": [{ type: Input },],
    "file": [{ type: Input },],
    "options": [{ type: Input },],
    "ready": [{ type: Output },],
    "fileChange": [{ type: Output },],
    "onResize": [{ type: HostListener, args: ['window:resize',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class MonacoEditorModule {
    /**
     * @param {?=} config
     * @return {?}
     */
    static forRoot(config = {}) {
        return {
            ngModule: MonacoEditorModule,
            providers: [
                MonacoEditorService,
                { provide: MONACO_EDITOR_OPTIONS, useValue: config.options || {} }
            ]
        };
    }
}
MonacoEditorModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [
                    MonacoEditorDirective
                ],
                exports: [
                    MonacoEditorDirective
                ]
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { MonacoEditorModule, MonacoEditorDirective, MonacoEditorService, COMPLETION_PROVIDERS, MONACO_EDITOR_OPTIONS };
//# sourceMappingURL=ngx-monaco.js.map
