import { __values } from 'tslib';
import { Observable, Subject } from 'rxjs';
import { InjectionToken, Injectable, Optional, Inject, NgZone, Directive, ElementRef, Input, Output, EventEmitter, HostListener, NgModule } from '@angular/core';
import { shareReplay, take, map, takeUntil, tap, debounceTime, filter, distinctUntilChanged } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

function fromDisposable(source) {
    return Observable.create(function (observer) {
        var disposable = source(function (e) {
            observer.next(e);
        });
        return function () { return disposable.dispose(); };
    });
}
var COMPLETION_PROVIDERS = new InjectionToken('Completion Provider');
var MONACO_EDITOR_OPTIONS = new InjectionToken('Monaco Editor Options');
var MonacoEditorService = /** @class */ (function () {
    function MonacoEditorService(completionProviders, editorOptions, zone) {
        this.completionProviders = completionProviders;
        this.editorOptions = editorOptions;
        this.zone = zone;
        this.onFileChange = new Subject();
        this.fileChange$ = this.onFileChange.asObservable();
        this.bootstrap$ = Observable.create(function (observer) {
            var script = document.createElement('script');
            script.type = 'text/javascript';
            script.src = 'libs/vs/loader.js';
            script.onload = function () {
                window.require.config({ paths: { vs: 'libs/vs' } });
                window.require(['vs/editor/editor.main'], function () {
                    observer.next();
                });
            };
            document.body.appendChild(script);
        }).pipe(shareReplay(1));
    }
    Object.defineProperty(MonacoEditorService.prototype, "editor", {
        get: function () {
            return this.monacoEditor;
        },
        enumerable: true,
        configurable: true
    });
    MonacoEditorService.prototype.registerCompletionProviders = function () {
        if (!this.completionProviders) {
            return;
        }
        try {
            for (var _a = __values(this.completionProviders), _b = _a.next(); !_b.done; _b = _a.next()) {
                var completionProvider = _b.value;
                monaco.languages.registerCompletionItemProvider(completionProvider.language, completionProvider);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
            }
            finally { if (e_1) throw e_1.error; }
        }
        var e_1, _c;
    };
    MonacoEditorService.prototype.registerModelChangeListener = function (file, model) {
        var _this = this;
        var destroy = fromDisposable(model.onWillDispose.bind(model)).pipe(take(1));
        fromDisposable(model.onDidChangeContent.bind(model))
            .pipe(map(function () { return model.getValue(); }), takeUntil(destroy))
            .subscribe(function (content) {
            _this.zone.run(function () {
                _this.onFileChange.next(Object.assign({}, file, { content: content }));
            });
        });
    };
    MonacoEditorService.prototype.disposeModels = function () {
        if (!window.monaco) {
            return;
        }
        try {
            for (var _a = __values(monaco.editor.getModels()), _b = _a.next(); !_b.done; _b = _a.next()) {
                var model = _b.value;
                model.dispose();
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
            }
            finally { if (e_2) throw e_2.error; }
        }
        var e_2, _c;
    };
    MonacoEditorService.prototype.load = function (container, options) {
        var _this = this;
        if (options === void 0) { options = {}; }
        var editorOptions = options.editor || this.editorOptions || {};
        return this.bootstrap$.pipe(tap(function () {
            _this.disposeModels();
            _this.monacoEditor = monaco.editor.create(container.nativeElement, Object.assign({ theme: options.theme }, editorOptions));
            _this.registerCompletionProviders();
            if (_this.file) {
                _this.open(_this.file);
            }
        }));
    };
    MonacoEditorService.prototype.open = function (file) {
        this.file = file;
        if (!this.monacoEditor) {
            return;
        }
        var uri = monaco.Uri.file(file.uri);
        var model = monaco.editor.getModel(uri);
        if (model) {
            if (file.language && model.getModeId() !== file.language) {
                model.dispose();
                model = undefined;
            }
            else {
                model.setValue(file.content);
            }
        }
        if (!model) {
            model = monaco.editor.createModel(file.content, file.language, uri);
            this.registerModelChangeListener(file, model);
        }
        this.monacoEditor.setModel(model);
    };
    MonacoEditorService.prototype.updateOptions = function (options) {
        if (this.monacoEditor) {
            this.monacoEditor.updateOptions(options);
        }
    };
    MonacoEditorService.prototype.setTheme = function (theme) {
        if (window.monaco) {
            monaco.editor.setTheme(theme);
        }
    };
    return MonacoEditorService;
}());
MonacoEditorService.decorators = [
    { type: Injectable },
];
MonacoEditorService.ctorParameters = function () { return [
    { type: Array, decorators: [{ type: Optional }, { type: Inject, args: [COMPLETION_PROVIDERS,] },] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [MONACO_EDITOR_OPTIONS,] },] },
    { type: NgZone, },
]; };
var MonacoEditorDirective = /** @class */ (function () {
    function MonacoEditorDirective(monacoEditorService, editorRef) {
        this.monacoEditorService = monacoEditorService;
        this.editorRef = editorRef;
        this.ready = new EventEmitter();
        this.fileChange = new EventEmitter();
        this.resize$ = new Subject();
        this.destroy$ = new Subject();
    }
    MonacoEditorDirective.prototype.onResize = function () {
        this.resize$.next();
    };
    MonacoEditorDirective.prototype.open = function (file) {
        this.monacoEditorService.open(file);
    };
    MonacoEditorDirective.prototype.ngOnInit = function () {
        var _this = this;
        this.monacoEditorService.load(this.editorRef, {
            theme: this.theme,
            editor: this.options
        }).subscribe(function () {
            _this.ready.emit();
        });
        this.monacoEditorService.fileChange$.pipe(takeUntil(this.destroy$)).subscribe(function (file) {
            _this.fileChange.emit(file);
        });
        this.resize$.pipe(filter(function () { return Boolean(_this.monacoEditorService.editor); }), map(function () { return ({ width: _this.editorRef.nativeElement.clientWidth, height: _this.editorRef.nativeElement.clientHeight }); }), distinctUntilChanged(function (a, b) { return a.width === b.width && a.height === b.height; }), debounceTime(50), takeUntil(this.destroy$)).subscribe(function (dimension) {
            _this.monacoEditorService.editor.layout(dimension);
        });
    };
    MonacoEditorDirective.prototype.ngOnDestroy = function () {
        this.destroy$.next();
    };
    MonacoEditorDirective.prototype.ngOnChanges = function (changes) {
        if (changes["options"]) {
            this.monacoEditorService.updateOptions(changes["options"].currentValue);
        }
        if (changes["theme"]) {
            this.monacoEditorService.setTheme(changes["theme"].currentValue);
        }
        if (changes["file"]) {
            this.open(changes["file"].currentValue);
        }
    };
    MonacoEditorDirective.prototype.ngAfterViewChecked = function () {
        this.resize$.next();
    };
    return MonacoEditorDirective;
}());
MonacoEditorDirective.decorators = [
    { type: Directive, args: [{
                selector: 'monaco-editor,[monaco-editor]'
            },] },
];
MonacoEditorDirective.ctorParameters = function () { return [
    { type: MonacoEditorService, },
    { type: ElementRef, },
]; };
MonacoEditorDirective.propDecorators = {
    "theme": [{ type: Input },],
    "file": [{ type: Input },],
    "options": [{ type: Input },],
    "ready": [{ type: Output },],
    "fileChange": [{ type: Output },],
    "onResize": [{ type: HostListener, args: ['window:resize',] },],
};
var MonacoEditorModule = /** @class */ (function () {
    function MonacoEditorModule() {
    }
    MonacoEditorModule.forRoot = function (config) {
        if (config === void 0) { config = {}; }
        return {
            ngModule: MonacoEditorModule,
            providers: [
                MonacoEditorService,
                { provide: MONACO_EDITOR_OPTIONS, useValue: config.options || {} }
            ]
        };
    };
    return MonacoEditorModule;
}());
MonacoEditorModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [
                    MonacoEditorDirective
                ],
                exports: [
                    MonacoEditorDirective
                ]
            },] },
];

export { MonacoEditorModule, MonacoEditorDirective, MonacoEditorService, COMPLETION_PROVIDERS, MONACO_EDITOR_OPTIONS };
//# sourceMappingURL=ngx-monaco.js.map
